declare type ParamsObject = Record<string, string | string[]>;
interface ParsedURL {
    protocol?: string;
    hostname?: string;
    port?: string;
    username?: string;
    password?: string;
    pathname: string;
    hash?: string;
    params?: ParamsObject;
}
declare function parseURL(input?: string): ParsedURL;
declare function parsePath(input?: string): ParsedURL;
declare function parseParams(paramsStr?: string): [string, string][];
declare function hasProtocol(inputStr: string): boolean;
declare function parsedParamsToObject(entries: [string, string][]): Record<string, string | string[]>;

declare class UFO implements URL {
    params: ParamsObject;
    hash: string;
    hostname: string;
    password: string;
    pathname: string;
    port: string;
    protocol: string;
    username: string;
    constructor(input?: string);
    get hasProtocol(): number;
    get isAbsolute(): number | boolean;
    get search(): string;
    get searchParams(): URLSearchParams;
    get host(): string;
    get origin(): string;
    get originWithAuth(): string;
    get auth(): string;
    get fullpath(): string;
    get href(): string;
    append(url: UFO): void;
    toJSON(): string;
    toString(): string;
}

declare function withoutTrailingSlash(input?: string): string;
declare function withTrailingSlash(input?: string): string;
declare function withoutLeadingSlash(input?: string): string;
declare function withLeadingSlash(input?: string): string;
declare function cleanDoubleSlashes(input?: string): string;
declare function createURL(input: string): UFO;
declare function normalizeURL(input: string): string;
declare function withParams(input: string, params: ParamsObject): string;
declare function getParams(input: string): ParamsObject;
declare function joinURL(base: string, ...input: string[]): string;

/**
 * Encode characters that need to be encoded on the path, search and hash
 * sections of the URL.
 *
 * @internal
 * @param text - string to encode
 * @returns encoded string
 */
declare function encode(text: string | number): string;
/**
 * Encode characters that need to be encoded on the hash section of the URL.
 *
 * @param text - string to encode
 * @returns encoded string
 */
declare function encodeHash(text: string): string;
/**
 * Encode characters that need to be encoded query values on the query
 * section of the URL.
 *
 * @param text - string to encode
 * @returns encoded string
 */
declare function encodeQueryValue(text: string | number): string;
/**
 * Like `encodeQueryValue` but also encodes the `=` character.
 *
 * @param text - string to encode
 */
declare function encodeQueryKey(text: string | number): string;
/**
 * Encode characters that need to be encoded on the path section of the URL.
 *
 * @param text - string to encode
 * @returns encoded string
 */
declare function encodePath(text: string | number): string;
/**
 * Encode characters that need to be encoded on the path section of the URL as a
 * param. This function encodes everything {@link encodePath} does plus the
 * slash (`/`) character.
 *
 * @param text - string to encode
 * @returns encoded string
 */
declare function encodeParam(text: string | number): string;
/**
 * Decode text using `decodeURIComponent`. Returns the original text if it
 * fails.
 *
 * @param text - string to decode
 * @returns decoded string
 */
declare function decode(text?: string | number): string;
declare function encodeSearchParam(key: string, val: string | string[]): string;
declare function encodeHost(name?: string): string;

export { ParamsObject, ParsedURL, UFO, cleanDoubleSlashes, createURL, decode, encode, encodeHash, encodeHost, encodeParam, encodePath, encodeQueryKey, encodeQueryValue, encodeSearchParam, getParams, hasProtocol, joinURL, normalizeURL, parseParams, parsePath, parseURL, parsedParamsToObject, withLeadingSlash, withParams, withTrailingSlash, withoutLeadingSlash, withoutTrailingSlash };
